# Generated by roxygen2: do not edit by hand

S3method(plot,surveysd)
S3method(print,summary.ipf)
S3method(print,surveysd)
S3method(summary,ipf)
export(calc.stError)
export(combine_factors)
export(computeFrac)
export(computeLinear)
export(computeLinearG1)
export(computeLinearG1_old)
export(demo.eusilc)
export(draw.bootstrap)
export(generate.HHID)
export(get.selection)
export(ipf)
export(ipf_step)
export(ipf_step_f)
export(ipf_step_ref)
export(kishFactor)
export(recalib)
export(rescaled.bootstrap)
export(weightedRatio)
export(weightedSum)
import(Rcpp)
import(data.table)
import(ggplot2)
importFrom("graphics","plot")
importFrom("laeken","weightedMedian")
importFrom("methods","formalArgs")
importFrom("methods","is")
importFrom("stats","as.formula")
importFrom("stats","formula")
importFrom("stats","na.omit")
importFrom("stats","quantile")
importFrom("stats","sd")
importFrom("stats","var")
importFrom("stats","xtabs")
importFrom("utils","capture.output")
importFrom("utils","data")
importFrom("utils","find")
importFrom("utils","head")
importFrom("utils","tail")
useDynLib(surveysd)
